(* Turbo-Pascal-Unit
   verschiedene Unterroutinen fr I/O
   J. Rathlev, Nov. 1995
      letzte nderung: Jan. 2000 *)

unit IOSub;

interface

uses Dos, Crt, NStrings, TPSub;

const
  DrvMax = 26;
  DefCursor = 1;
  ActCursorTyp : byte=1;

type
  NamExtStr = string[12];       (* Dateiname mit Erweiterung *)
  NormString = string[80];
  DrvLstArr = array [1..DrvMax] of byte;

{ ---------------------------------------------------------------- }
(* Name und Pfad des Programms *)
procedure GetPrgName (var Pfad : DirStr;
                      var Name : NameStr);

(* Pfadnamen aus vollst. Dateinamen *)
function GetPath (Pfad : PathStr) : DirStr;

(* Pfad in einem Dateinamen ndern *)
function NewPath (Pfad,Name : string) : string;

(* Name aus vollst. Dateinamen *)
function GetName (Pfad : PathStr) : NameStr;

(* Name mit Erweiterung aus vollst. Dateinamen *)
function GetNameExt (Pfad : PathStr) : NamExtStr;

(* Datei in anderen Pfaden (DOS-Befehl "PATH") suchen *)
function SearchPath (name : string) : string;

(* DOS-Pfad *)
function EnvPath : string;

(* Pfad aus dem Dateinamen entfernen, wenn er gleich "Pfad" ist *)
function RemPath (Pfad,Name : string) : string;

{ ---------------------------------------------------------------- }
(* akt. Zeit in String umsetzen *)
function ActTimeToStr : string;

(* akt. Datum in String umsetzen *)
function ActDateToStr : string;

(* akt. Datum mit Monatsnamen in String umsetzen *)
function ActDateToLongStr: string;

{ ---------------------------------------------------------------- }
(* lesen des Erzeugungsdatums einer Datei *)
procedure FDate (var f;
                 var Tag,Monat,Jahr : word);

(* prfen, ob eine Datei vom Text-Typ vorhanden ist *)
function FileExist (var f : text) : boolean;

{ ---------------------------------------------------------------- }
(* Lesen der DOS-Versionsnummer *)
function GetDosVersion : word;

(* Verzeichnis wechseln *)
function ChangeDir (var Dir : OpenString) : boolean;

(* Verzeichnis erzeugen *)
function MakeDir (Dir : OpenString) : boolean;
(* Ergebnis = true  : Erzeugung erfolgreich
            = false : Fehler *)

(* Tabelle der angeschlossenen Laufwerke ermitteln *)
procedure GetLoggedDrive (var DrvAnz : byte;
                          var DrvLst : DrvLstArr);

(* Laufwerk wechseln *)
procedure SetCurrentDrive (Drive: byte);

(* aktuelle Laufwerksnummer *)
function GetCurrentDrive: byte;

{ ---------------------------------------------------------------- }
(* Textstring auf Bildschirm ausgeben *)
procedure WrString (s   : string;
                    x,y : byte);

(* String von Tastatur einlesen *)
procedure RdString (var s    : OpenString;
                    SLen,x,y : byte;
                    var abbr : boolean);

(* String zentriert zwischen xl und xr in Zeile y ausgeben *)
procedure PutCenter (titel     : string;
                     xl,xr,y   : byte);

(* Rahmen auf Bildschirm zeichnen *)
procedure Rahmen (xol,yol,xur,yur,ym : byte;
                  titel              : string);

{ ---------------------------------------------------------------- }
(* Cursorform verndern *)
procedure ChgCursor (CursTyp : byte);

(* Textseiten (0..3) umschalten *)
procedure TextPage (page : byte);

(* aktive Textseite (0..3) ermitteln *)
function GetActPage : byte;

(* Cursorposition setzen *)
procedure SetCursor (seite,x,y : byte);

(* Cursorposition lesen *)
procedure GetCursor (seite   : byte;
                     var x,y : byte);

(* bis Zeilende lschen *)
procedure ScrClEol (Seite : byte);

(* String auf Bildschirm schreiben mit CrLf *)
procedure ScrStrLn (seite : byte;
                    text  : string);

(* String auf Bildschirm schreiben *)
procedure ScrStr (seite : byte;
                  text  : string);

(* Zahl auf Bildschirm schreiben *)
procedure ScrInt (seite : byte;
                  n,nd  : integer);

{ ---------------------------------------------------------------- }
implementation
var
  DosVers : word;
{ --------------------------------------------------------------- }
(* Name und Pfad des Programms *)
procedure GetPrgName (var Pfad : DirStr;
                      var Name : NameStr);
var
  Ext : ExtStr;
begin
  FSplit(ParamStr(0),Pfad,Name,Ext);
  end;

{ --------------------------------------------------------------- }
(* Pfadnamen aus vollst. Dateinamen *)
function GetPath (Pfad : PathStr) : DirStr;
var
  Dir  : DirStr;
  Name : NameStr;
  Ext  : ExtStr;
  dl   : byte absolute Dir;
begin
  FSplit (Pfad,Dir,Name,Ext);
  if Dir[dl]='\' then dec(dl);
  GetPath:=Dir;
  end;

{ --------------------------------------------------------------- }
(* Pfad in einem Dateinamen ndern *)
function NewPath (Pfad,Name : string) : string;
var
  ds       : DirStr;
  ns       : NameStr;
  es       : ExtStr;
begin
  FSplit (Name,ds,ns,es);
  NewPath:=SetDir(Pfad)+ns+es;
  end;

{ --------------------------------------------------------------- }
(* Name aus vollst. Dateinamen *)
function GetName (Pfad : PathStr) : NameStr;
var
  Dir  : DirStr;
  Name : NameStr;
  Ext  : ExtStr;
begin
  FSplit (Pfad,Dir,Name,Ext);
  GetName:=Name;
  end;

{ ---------------------------------------------------------------- }
(* Name mit Erweiterung aus vollst. Dateinamen *)
function GetNameExt (Pfad : PathStr) : NamExtStr;
var
  Dir  : DirStr;
  Name : NameStr;
  Ext  : ExtStr;
begin
  FSplit (Pfad,Dir,Name,Ext);
  GetNameExt:=Name+Ext;
  end;

{ ---------------------------------------------------------------- }
function EnvPath : string;
{ Diese Funktion liefert den Path-String des Programm-Environment zurck }
begin
  EnvPath:=GetEnv('PATH');
  end;

{ ---------------------------------------------------------------- }
function SearchPath (name : string) : string;
{BESCHREIBUNG:
   Diese Funktion ergnzt den Namen einer Datei durch einen Pfad
   der aus der DOS-Programmumgebung ermittelt wurde.

PARAMETER:
   NAME  (Eingabe)        - Name der Datei nach der gesucht wird
   SearchPath (Ausgabe)   - mit Pfadnamen ergnzter Dateiname, wenn
                            gefunden
                          = '', wenn nicht gefunden

BEISPIELHAFTER AUFRUF:
   neuername:=SearchPath (name);
   ---------------------------------------------------------------- }
begin
  SearchPath:=FSearch(Name,EnvPath);
  end;

{ ---------------------------------------------------------------- }
(* Pfad aus dem Dateinamen entfernen, wenn er gleich "Pfad" ist *)
function RemPath (Pfad : string;
                  Name : string) : string;
var
  pl : byte absolute Pfad;
  ds : DirStr;
  dl : byte absolute ds;
  ns : NameStr;
  es : ExtStr;
begin
  FSplit (Name,ds,ns,es);
  if (ds[dl]='\') and (Pfad[pl]<>'\') then Pfad:=Pfad+'\';
  if OEMUpString(Pfad)=ds then RemPath:=ns+es else RemPath:=Name;
  end;

{ ------------------------------------------------------------------- }
(* akt. Zeit in String umsetzen *)
function ActTimeToStr : string;
var
  Hour,Minu,Sek,SekH: word;      (* aktuelle Zeit *)
begin
  GetTime (Hour,Minu,Sek,SekH);
  ActTimeToStr:=StrInt(Hour,2)+TmMark+StrInt(Minu,2)+TmMark+StrInt(Sek,2);
  end;

{ ------------------------------------------------------------------- }
(* akt. Datum in String umsetzen *)
function ActDateToStr : string;
var
  Y,M,D,DoW : word;
begin
  GetDate (Y,M,D,DoW); Y:=Y mod 100;
  ActDateToStr:=StrInt(D,2)+Punkt+StrInt(M,2)+Punkt+ZStrInt(Y,2);
  end;

{ ---------------------------------------------------------------------- }
(* akt. Datum mit Monatsnamen in String umsetzen *)
function ActDateToLongStr : string;
const
  monat : array [1..12] of string[3]=
           ('Jan','Feb','Mrz','Apr','Mai','Jun',
            'Jul','Aug','Sep','Okt','Nov','Dez');
var
  k1,k2,k3,k4 : word;
begin
  GetDate (k1,k2,k3,k4);
  ActDateToLongStr:=StrInt(k3,2) + '.' + monat[k2] + '.' + StrInt(k1,4);
  end;

{-------------------------------------------------------------------}
(* lesen des Erzeugungsdatums einer Datei *)
procedure FDate (var f;
                 var Tag,Monat,Jahr : word);
var
  PT      : longint;
  DatZeit : datetime;

begin
  GetFTime (file(f),PT); UnpackTime (PT,DatZeit);
  Tag:=DatZeit.Day; Monat:=DatZeit.Month;
  Jahr:=DatZeit.Year mod 1900;
  end;

{ ---------------------------------------------------------------- }
function FileExist (var f : text) : boolean;
{BESCHREIBUNG:
   Diese Funktion prft, ob eine Datei vom text-Typ vorhanden ist
PARAMETER:
   F     (Eingabe)       - Dateivariable
   FileExist (Ausgabe)  - True oder False
BEISPIELHAFTER AUFRUF:
   if FileExist (f) then ...
   ---------------------------------------------------------------- }
begin
  (*$i-*) reset (f) (*$i+*);
  FileExist:=ioresult=0;
  end;

{-------------------------------------------------------------------}
(* Lesen der DOS-Versionsnummer *)
function GetDosVersion : word;
var
  reg  : registers;
begin
  with reg do begin
    ah:=$30;
    MsDos(reg);
    GetDosVersion:=Swap(ax);
    end;
  end;

{-------------------------------------------------------------------}
(* Verzeichnis wechseln *)
function ChangeDir (var Dir : OpenString) : boolean;
(* Ergebnis = true  : Wechsel erfolgreich
            = false : Fehler
   Der vollstndige Verzeichnisname wird zurckgegeben *)
var
  dl : byte absolute Dir;
begin
  if (Dir[dl]='\') and (dl>1) and (Dir[dl-1]<>':') then dec(dl);
  Dir:=OEMUpString(Dir);
  (*$i-*) ChDir(Dir)(*$i+*);
  if IOResult=0 then begin
    GetDir (0,Dir); ChangeDir:=true;
    end
  else ChangeDir:=false;
  end;

{-------------------------------------------------------------------}
(* Verzeichnis erzeugen *)
function MakeDir (Dir : OpenString) : boolean;
(* Ergebnis = true  : Erzeugung erfolgreich
            = false : Fehler *)
var
  dl : byte absolute Dir;
begin
  if (Dir[dl]='\') and (dl>1) and (Dir[dl-1]<>':') then dec(dl);
  Dir:=OEMUpString(Dir);
  (*$i-*) MkDir(Dir)(*$i+*);
  MakeDir:=IOResult=0;
  end;

{ --------------------------------------------------------------- }
(* Tabelle der angeschlossenen Laufwerke (einschl. Netzlaufwerke)
   ermitteln *)
procedure GetLoggedDrive (var DrvAnz : byte;
                          var DrvLst : DrvLstArr);
type
  PtrPart = record
    Ofs,Seg : word;
    end;

  DPB = record
    DrvNr : byte;
    Dummy : array [1..$1C] of byte;
    end;

var
  reg  : registers;
  p    : ^pointer;
  dp   : ^DPB;
  b    : byte;
  dev  : array [0..15] of char;
  nwp  : array [0..127] of char;
  ndx  : word;
begin
  with reg do begin
  (* Anzahl der Diskettenlaufwerke *)
    Intr ($11,reg);
    DrvAnz:= ((AX and $C0) shr 6) + 1;
    for b:=1 to DrvAnz do DrvLst[b]:=b;
    end;
  (* Festplattenlaufwerke *)
  b:=succ(DrvAnz);
  repeat
    with reg do begin
      ah:=$32; dl:=b;
      MsDos (reg);
      dp:=ptr(ds,bx); inc(b);
      if (al<>$FF) then with dp^ do begin
        inc(DrvAnz);
        DrvLst[DrvAnz]:=succ(DrvNr);
        if DosVers>=$500 then p:=@Dummy[$19] else p:=@Dummy[$18];
        dp:=p^;
        end;
      end;
    until (PtrPart(dp).ofs=$FFFF) or (b>26);
  (* Netzlaufwerke *)
  ndx:=0;
  with reg do repeat
    ax:=$5f05; bx:=ndx; cx:=0;
    ds:=seg(dev); si:=ofs(dev);
    es:=seg(nwp); di:=ofs(nwp);
    MsDos (reg);
    if Flags and 1=0 then begin
      inc(DrvAnz);
      DrvLst[DrvAnz]:=ord(dev[0])-64;
      end;
    inc(ndx);
    until Flags and $1<>0 ;
  end;

{ --------------------------------------------------------------- }
(* erweiterte DOS-System-Routinen
   nach Jamsa/Nameroff, Turbo Pascal Programmbibliothek
   Turbo 4 - Version *)
{ --------------------------------------------------------------- }
procedure SetCurrentDrive (DRIVE: byte);
{
BESCHREIBUNG:
   Dieses Unterprogramm bestimmt das aktuelle voreingestellte Lauf-
   werk.

PARAMETER:
   DRIVE (Eingabe) - Gewnschtes voreingestelltes Laufwerk

BENTIGTE TYPEN:
   REGISTERS - Wird von allen ROM-BIOS und DOS-Unterbrechungen benutzt

BEISPIELHAFTER AUFRUF:
   SetCurrentDrive (1);

ANMERKUNGEN:
   Laufwerke werden aufgrund folgender Zuordnung ausgewhlt:
   A = 1, B = 2, C = 3.
   Unzulssige Angaben ndern nicht das aktuelle Laufwerk.

   ---------------------------------------------------------------- }
var
  REGS: REGISTERS;  { AX, BX, CX, DX, BP, SI, DI, DS, ES, FLAGS }
begin
  with REGS do begin
      AX := $E00;        { DOS Funktionscode }
      DX := pred(DRIVE);
      MSDos (REGS);
    end;
end;

{ ---------------------------------------------------------------- }
function GetCurrentDrive: byte;
{
BESCHREIBUNG:
   Dieses Unterprogramm bergibt das aktuelle Laufwerk.
BERGEBENER WERT:
   Byte-Wert, der das aktuelle Laufwerk angibt.
BENTIGTE TYPEN:
   REGISTERS - Wird von allen ROM-BIOS und DOS-Unterbrechungen benutzt
BEISPIELHAFTER AUFRUF:
   if (GetCurrentDrive <> 1) then
     SetCurrentDrive (1);
ANMERKUNGEN:
   Laufwerke werden  mit A = 1, B = 2, C = 3 bezeichnet.
   ---------------------------------------------------------------- }
var
  REGS: REGISTERS;  { AX, BX, CX, DX, BP, SI, DI, DS, ES, FLAGS }
begin
  with REGS do begin
{ Setze AX, um das aktuelle Laufwerk anzusprechen }
      AX := $1900;
      MSDos (REGS);
      GetCurrentDrive := succ(Lo(AX));
    end;
end;

{------------------------------------------------------------------}
const
(* Tastaturcodes einiger Tasten *)
  bell = #7;    (* Bell *)
  bs = #8;      (* Backspace *)
  tab = #9;     (* Tab *)
  ret = #13;    (* Carr.Return *)
  ctt = ^T;     (* Ctrl. T *)
  cty = ^Y;     (* Ctrl. Y *)
  ctr = ^R;     (* Ctrl. R *)
  esc = #27;    (* ESc-Code *)
  links = #75;  (* Linkspfeil *)
  rechts = #77; (* Rechtspfeil *)
  home = #71;   (* Home *)
  ende = #79;   (* End *)
  ins  = #82;   (* Insert *)
  del  = #83;   (* delete *)
  cli  = #115;  (* Ctrl. Linkspfeil *)
  cre  = #116;  (* Ctrl. Rechtspfeil *)

{------------------------------------------------------------------}
(* Textstring auf Bildschirm ausgeben
   s   : Textstring
   x,y : Bildschirmkoord. des ersten auszugebenden Zeichens *)
procedure WrString (s   : string;
                    x,y : byte);
begin
  if x<>0 then gotoxy(x,y); write (s);
  end;

{------------------------------------------------------------------}
(* String von Tastatur einlesen *)
procedure RdString (var s    : OpenString;
                    SLen,x,y : byte;
                    var abbr : boolean);
(* SLen  : max. Lnge des Strings
   x,y = Spalte, Zeile
   x = 0 : an momentaner Pos.
   y > 0 : alten String berschreiben
   y < 0 : an alten String anhngen *)
var
  c        : char;
  ns       : string;
  sl       : byte absolute ns;
  cp,i,j,k : integer;
  first,
  insmd,ow : boolean;

  (* Zeichen in String einfgen *)
  procedure InsChar (c : char);
  begin
    if cp>SL then begin
      if SL<SLen then ns:=ns+c else write (^G);
      end
    else if insmd then begin
      if SL<SLen then insert (c,ns,cp) else write (^G);
      end
    else begin
      ns[cp]:=c; if cp=SLen then write (^G);
      end;
    if cp<SLen then inc(cp);
    end;

  (* String lschen *)
  procedure ClrStr;
  begin
    ns:=''; cp:=1;
    end;

begin
  ns:=s;
  if y<0 then begin
    y:=-y; ow:=false;
    end
  else ow:=true;
  if x=0 then begin
    x:=wherex; y:=wherey;
    end;
  ClrEol; WrString (ns,x,y); gotoxy (x+sl,y);
  c:=ReadKey; First:=ow;
  cp:=succ(Sl); insmd:=true; ChgCursor(1);
  while (c<>ret) and (c<>Esc) do begin
    case c of
    bs : begin
         if cp>1 then begin
           dec(cp); delete (ns,cp,1);
           end;
         end;
    cty : ClrStr;
    ctr : begin
         ns:=s; cp:=succ(length(s));
         end;
    ctt : begin   (* Wort lschen *)
         i:=cp;
         if ns[i]=Space then while (i<=SL) and (ns[i]=Space) do inc(i)
         else while (i<=SL) and (ns[i]<>Space) do inc(i);
         delete (ns,cp,i-cp);
         end;
    #0 : begin
         c:=ReadKey;
         case c of
         links  : if cp>1 then dec(cp);
         rechts : if (cp<=SL) and (cp<SLen) then inc(cp);
         ins    : insmd:=not insmd; (* toggle insert-mode *)
         del    : delete (ns,cp,1);
         home   : cp:=1;
         ende   : if SL<SLen then cp:=succ(SL) else begin
                  cp:=SLen; write (^G);
                  end;
         cli    : if cp>1 then begin  (* Wort links *)
                  if (cp>SL) or (ns[pred(cp)]=Space) then dec(cp);
                  while (cp>1) and (ns[cp]=Space) do dec(cp);
                  while (cp>1) and (ns[cp]<>Space) do dec(cp);
                  if ns[cp]=Space then inc(cp);
                  end;
         cre    : begin   (* Wort rechts *)
                  while (cp<=SL) and (ns[cp]<>Space) do inc(cp);
                  while (cp<=SL) and (ns[cp]=Space) do inc(cp);
                  if cp>SL then cp:=SL;
                  end;
           end;
         end
       else begin
         if First then ClrStr; InsChar (c);
         end;
      end;
    First:=false;
    if insmd then ChgCursor(1) else ChgCursor(2);
    WrString (ns,x,y); ClrEol; gotoxy (pred(x+cp),y);
    c:=ReadKey;
    end;
  if c=Esc then abbr:=true
  else begin
    abbr:=false; s:=ns;
    end;
  WrString (s,x,y);
  end;

{------------------------------------------------------------------}
(* String zentriert zwischen xl und xr in Zeile y ausgeben *)
procedure PutCenter (titel     : string;
                     xl,xr,y   : byte);
begin
  gotoxy (xl+(xr-xl-length(titel)) div 2,y); write (titel)
  end;

{------------------------------------------------------------------}
(* Rahmen auf Bildschirm zeichnen *)
procedure Rahmen (xol,yol,xur,yur,ym : byte;
                  titel              : string);
(* x und y Koord. des eingeschl. Fensters und des Teilstrichs (ym)
   oben links und unten rechts in Zeichen und Zeilen *)
var
  x1,x2,i : byte;
begin
  if xol<3 then xol:=1 else xol:=xol-2;
  if xur>77 then xur:=79 else xur:=xur+2;
  gotoxy (xol,pred(yol)); write ('');
  for i:=succ(xol) to pred(xur) do write ('');
  write ('');
  for i:=yol to yur do begin
    gotoxy (xol,i); write ('','':xur-xol);
    end;
  gotoxy (xol,succ(yur)); write ('');
  for i:=succ(xol) to pred(xur) do write ('');
  write ('');
  if ym<>0 then begin
    gotoxy (xol,succ(ym)); write ('');
    for i:=succ(xol) to pred(xur) do write ('');
    write ('');
    end;
  PutCenter (titel,xol,xur,pred(yol));
  end;

{------------------------------------------------------------------}
(* Textmodus abfragen
   Monochrom = 7 *)
function GetTextMode : integer;
var
  reg : registers;
begin
  reg.ah:=15;
  INTR ($10,reg);
  GetTextMode:=reg.al;
  end;

{------------------------------------------------------------------}
(* Cursorform verndern *)
procedure ChgCursor (CursTyp : byte);
const
  CursTypAnz = 2;
type
  CursArr = array [1..2,1..CursTypAnz] of byte;
const
  (* Normal / Monochrom *)
  CursAnf : CursArr = ((6,1),(11,1));
  CursEnd : CursArr = ((7,7),(12,12));
var
  reg : registers;
  n   : byte;
begin
  ActCursorTyp:=CursTyp;
  with reg do if CursTyp>CursTypAnz then begin   (* Cursor Off *)
{    ah:=2; bh:=GetActPage and 3;
    dx:=$1900;}
    ah:=1; ch:=$20; cl:=0;
    end
  else begin
    ah:=1;
    if CursTyp<1 then CursTyp:=1;
    if GetTextMode=7 then n:=2 else n:=1;
    ch:=CursAnf[n,CursTyp];
    cl:=CursEnd[n,CursTyp];
    end;
  INTR($10,reg);
  end;

{ ---------------------------------------------------------------- }
(* Textseiten (0..3) umschalten *)
procedure TextPage (page : byte);
var
  reg : registers;
begin
  reg.al:=page mod 4; reg.ah:=5;
  intr ($10,reg);
  end;

{ ---------------------------------------------------------------- }
(* aktive Textseite (0..3) ermitteln *)
function GetActPage : byte;
var
  reg : registers;
begin
  reg.ah:=$F;
  intr ($10,reg);
  GetActPage:=reg.bh;
  end;

{ ---------------------------------------------------------------- }
(* Cursorposition setzen *)
procedure SetCursor (seite,x,y : byte);
var
  reg : registers;
begin
  with reg do begin
    ah:=2; dh:=(y-1) mod 25; dl:=(x-1) mod 80;
    bh:=seite and 3;
    end;
  intr ($10,reg);
  end;

{ ---------------------------------------------------------------- }
(* Cursorposition lesen *)
procedure GetCursor (seite   : byte;
                     var x,y : byte);
var
  reg : registers;
begin
  with reg do begin
    ah:=3; bh:=seite mod 4;
    intr ($10,reg);
    y:=dh+1; x:=dl+1;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Bildschirmfenster nach oben rollen *)
procedure ScrollUp (x1,y1,x2,y2,n : byte);
var
  reg : registers;
begin
  with reg do begin
    ch:=y1-1; cl:=x1-1; dh:=y2-1; dl:=x2-1; bh:=7;
    al:=n; ah:=6;
    end;
  intr ($10,reg);
  end;

{ ---------------------------------------------------------------- }
(* bis Zeilende lschen *)
procedure ScrClEol (Seite : byte);
var
  i,x,y : byte;
  reg   : registers;
begin
  GetCursor (Seite,x,y);
  with reg do begin
    ah:=9; al:=32; bh:=Seite; bl:=7; cx:=81-x;
    end;
  intr ($10,reg);
  SetCursor (Seite,x,y);
  end;

{ ---------------------------------------------------------------- }
(* Zeichen und Attribut an Cursorpos. schreiben *)
procedure ScrWrt (seite   : byte;
                  zeichen : char);
var
  reg : registers;
begin
  with reg do begin
    ah:=$E; al:=ord(zeichen); bl:=0;
    bh:=seite mod 4;
    end;
  intr ($10,reg);
  end;

{ ---------------------------------------------------------------- }
(* String auf Bildschirm schreiben *)
procedure ScrStr (seite : byte;
                  text  : string);
var
  i,x,y : byte;
begin
  for i:=1 to length(text) do ScrWrt(seite,text[i]);
  end;

{ ---------------------------------------------------------------- }
(* String auf Bildschirm schreiben mit CrLf *)
procedure ScrStrLn (seite : byte;
                    text  : string);
const
  CrLf = #13+#10;
var
  x,y : byte;
begin
  ScrStr (seite,text+CrLf);
  end;

{ ---------------------------------------------------------------- }
(* Zahl auf Bildschirm schreiben *)
procedure ScrInt (seite : byte;
                  n,nd  : integer);
var
  zahl : string[20];
begin
  str (n:nd,zahl); ScrStr (seite,zahl);
  end;

{===========================================================================}
begin
  DosVers:=GetDosVersion;
  ChgCursor (DefCursor);
end.
