(* spezielle mathematische Funktionen
   Turbo Pascal Unit
   J. Rathlev, Jun. 1989
      letzte nderung: Jun. 2001 *)

unit Math;

interface

type
  EinhStr = string[5];

const
  Pi180 = Pi/180.0;
  Maxdouble : double = 1E300;

function AInt (x : double) : double;

function RMod (x,y : double) : double;

function Sign (x,y : double) : double;

function Lg (x : double) : double;

function Ld (x : double) : double;

function Pwr (x,y : double) : double;

function Kub (x : double) : double;

function Tan (x : double) : double;

function ArcTan1 (x,y : double) : double;

function ArcTan2 (x,y : double) : double;

function ArcTanB (x,y : double) : double;

function Min (x,y : integer) : integer;

function Max (x,y : integer) : integer;

function RMin (x,y : double) : double;

function RMax (x,y : double) : double;

(* Normalisieren eines Wertes in den Bereich 0.1 .. 999.9 *)
procedure Normal (var Value,Factor : double;
                  var Prefix       : char);

(* Zahlenwert mit nachfolgendem Einheitenvorsatz erzeugen *)
function PrefixVal (Value : double;
                    n,nd  : integer;
                    Einh  : EinhStr) : string;

(* Zahlenwert aus String mit nachfolgendem Einheitenvorsatz erzeugen *)
function ValPrefix (ValStr    : string;
                    var Value : double) : boolean;

implementation
var
  ln2,ln10 : double;

(* ----------FUNCTION--AINT--------------------------IAP-KIEL----------
   Bestimmung des nchstkleineren ganzzahligen Anteils einer Zahl
*)
function AInt (x : double) : double;
var
  xx  : double;
begin
  xx:=int(x);
  if (x<0.0) and (x<>xx) then xx:=xx-1.0;
  aint:=xx;
  end;

(* ----------FUNCTION--RMOD--------------------------IAP-KIEL----------
   Divisionsrest bei Fliekommazahlen
*)
function RMod (x,y : double) : double;
begin
  RMod:=x-int(x/y)*y;
  end;

(* ----------FUNCTION--SIGN--------------------------IAP-KIEL----------
  (Vorzeichen von arg2) * arg1
*)
function Sign (x,y : double) : double;
begin
  if y<0.0 then sign:=-x else if y>0.0 then sign:=x else sign:=0.0;
  end;

(* ----------FUNCTION--LG----------------------------IAP-KIEL----------
  Dekadischer Logarithmus
*)
function Lg (x : double) : double;
begin
  lg:=ln(x)/ln10;
  end;

(* ----------FUNCTION--Ld----------------------------IAP-KIEL----------
  Dualer Logarithmus
*)
function Ld (x : double) : double;
begin
  Ld:=ln(x)/ln2;
  end;

(* ----------FUNCTION--PWR---------------------------IAP-KIEL----------
  allgemeine Exponentialfunktion x ^ y
*)
function Pwr (x,y : double) : double;
begin
  pwr:=exp (y*ln(x));
  end;

(* ----------FUNCTION--KUB---------------------------IAP-KIEL----------
   Kubik (x hoch 3) *)
function Kub (x : double) : double;
begin
  Kub:=sqr(x)*x;
  end;

(* ----------FUNCTION--TAN---------------------------IAP-KIEL----------
  Tangens
  *)
function Tan (x : double) : double;
var
  c : double;
begin
  c:=cos(x);
  if c<>0.0 then Tan:=sin(x)/c
  else Tan:=Maxdouble;
  end;

(* ----------FUNCTION--ArcTan1-----------------------IAP-KIEL----------
   ArcTan mit Quadrantenbestimmung, Winkel von 0 .. 360 Grad
   linksweisend gegen Ost (math. pos. Sinn) *)
function ArcTan1 (x,y : double) : double;
var
  z : double;
begin
  if abs(x)<abs(y) then begin
    z:=90.0-ArcTan (x/y)/Pi180;
    if y<0.0 then z:=z+180.0;
    end
  else begin
    if x=0.0 then z:=0.0
    else z:=ArcTan (y/x)/Pi180;
    if x<0.0 then z:=z+180.0;
    end;
  if z<0.0 then z:=z+360.0;
  ArcTan1:=z;
  end;

(* ----------FUNCTION--ArcTan2-----------------------IAP-KIEL----------
   ArcTan mit Quadrantenbestimmung, Winkel von 0 .. 360 Grad
   rechtsweisend gegen Nord
*)
function ArcTan2 (x,y : double) : double;
var
  z : double;
begin
  if abs(x)<abs(y) then begin
    z:=ArcTan (x/y)/Pi180;
    if y<0.0 then z:=z+180.0
    else if x<0.0 then z:=z+360.0;
    end
  else begin
    if x=0.0 then z:=0.0
    else z:=90.0-ArcTan (y/x)/Pi180;
    if x<0.0 then z:=z+180.0;
    end;
  ArcTan2:=z;
  end;

(* ----------FUNCTION--ArcTan-----------------------IAP-KIEL----------
   ArcTan mit Quadrantenbestimmung, Winkel in Bogenma
   +/- Pi im math. pos. Sinn *)
function ArcTanB (x,y : double) : double;
var
  z : double;
begin
  if abs(x)<abs(y) then begin
    z:=0.5*Pi-ArcTan (x/y);
    if y<0.0 then z:=z+Pi
    else if x<0.0 then z:=z+2*Pi;
    end
  else begin
    if x=0.0 then z:=0.0
    else z:=ArcTan (y/x);
    if x<0.0 then z:=z+Pi;
    end;
  if z<=-Pi then z:=z+2*Pi;
  if z>Pi then z:=z-2*Pi;
  ArcTanB:=z;
  end;

(* ----------FUNCTION--MIN---------------------------IAP-KIEL----------
   Mininum zweier Integerzahlen
*)
function min (x,y : integer) : integer;
begin
  if x<y then min:=x else min:=y;
  end;

(* ----------FUNCTION--MAX---------------------------IAP-KIEL----------
   Maximum zweier Zahlen
*)
function max (x,y : integer) : integer;
begin
  if x<y then max:=y else max:=x;
  end;

(* ----------FUNCTION--RMIN--------------------------IAP-KIEL----------
   Mininum zweier Realzahlen
*)
function rmin (x,y : double) : double;
begin
  if x<y then rmin:=x else rmin:=y;
  end;

(* ----------FUNCTION--RMAX--------------------------IAP-KIEL----------
   Maximum zweier Realzahlen
*)
function rmax (x,y : double) : double;
begin
  if x<y then rmax:=y else rmax:=x;
  end;

(* ----------PROCEDURE-NORMAL------------------------IAP-KIEL---------- *)
const
  PfAnz = 10;
  Pfx   : array [1..PfAnz] of char = ('f','p','n','','m',' ','k','M','G','T');
  PfFak : array [0..PfAnz] of double = (1.0,1E15,1E12,1E9,1E6,1E3,
                                        1.0,1E-3,1E-6,1E-9,1E-12);
(* Normalisieren eines Wertes in den Bereich 0.1 .. 999.9 *)
procedure Normal (var Value,Factor : double;
                  var Prefix       : char);
var
  AVal : double;
  k    : integer;
begin
  AVal:=abs(Value);
  if (AVal<1E-15) or (AVal>1E15) then k:=6
  else begin
    k:=1;
    if (AVal>=1.0E-13) then inc(k);
    if (AVal>=1.0E-10) then inc(k);
    if (AVal>=1.0E-7) then inc(k);
    if (AVal>=1.0E-4) then inc(k);
    if (AVal>=1.0E-1) then inc(k);
    if (AVal>=0.999E3) then inc(k);
    if (AVal>=0.999E6) then inc(k);
    if (AVal>=0.999E9) then inc(k);
    if (AVal>=0.999E12) then inc(k);
    end;
  Factor:=PfFak[k];
  Value:=Value*Factor;
  Prefix:=Pfx[k];
  end;

(* ----------FUNCTION--PREFIXVAL---------------------IAP-KIEL---------- *)
(* Zahlenwert mit nachfolgendem Einheitenvorsatz erzeugen *)
function PrefixVal (Value   : double;
                    n,nd    : integer;
                    Einh    : EinhStr) : string;
(* n    = Anzahl der Stellen
        = 0: automatische Stellenanpassung
   nd   = Anzahl der Stellen hinter dem Komma
          <0 : automatisch max. -(nd-1) Stellen hinter dem Komma
   Einh = nachfolgendes Einheitenzeichen *)
var
  s    : string;
  pf   : char;
  f    : double;
  i    : integer;
begin
  Normal (Value,f,pf);
  if nd<0 then begin
    nd:=abs(nd)-3;
    if abs(Value)<100.0 then inc(nd);
    if abs(Value)<10.0 then inc(nd);
    if abs(Value)<1.0 then inc(nd);
    if abs(Value)<0.1 then nd:=0;
    if nd<0 then nd:=0;
    end;
  if n=0 then begin
    n:=3+nd; if nd>0 then inc(n);
    if Value<100.0 then dec(n);
    if Value<10.0 then dec(n);
    end;
  if Value<0.0 then inc(n);
  str (Value:n:nd,s);
  if Einh='' then begin
    if pf<>' ' then PrefixVal:=s+pf
    else PrefixVal:=s;
    end
  else if pf<>' ' then PrefixVal:=s+' '+pf+Einh
    else PrefixVal:=s+' '+Einh+' ';
  end;

(* ----------FUNCTION--VALPREFIX---------------------IAP-KIEL---------- *)
(* Zahlenwert aus String mit nachfolgendem Einheitenvorsatz erzeugen *)
function ValPrefix (ValStr  : string;
                    var Value : double) : boolean;
(* PrefixVal = true : ok
             = false : Fehler bei der Konvertierung *)
var
  lval : byte absolute ValStr;
  k,ic : integer;
  c    : char;
begin
  (* Prfe letztes Zeichen *)
  c:=ValStr[lval]; Value:=0.0; k:=0;
  if (c<'0') or (c>'9') then begin
    k:=1;
    while (k<=PfAnz) and (Pfx[k]<>c) do inc(k);
    end;
  if k<=PfAnz then begin
    if k>0 then begin
      dec(lval);
      while (lval>0) and (ValStr[lval]=' ') do dec(lval);
      end;
    val (ValStr,Value,ic);
    if ic=0 then begin
      Value:=Value/PfFak[k]; ValPrefix:=true;
      end
    else ValPrefix:=false;
    end
  else ValPrefix:=false;
  end;

(* ----------Initialisierung-------------------------IAP-KIEL---------- *)
begin
  (* Initialisierung der Konstanten *)
  ln10:=ln(10); ln2:=ln(2);
  end.
