(* Routinen zum Lesen und Schreiben von Bytes, Zeilen, etc.
   mit Hilfe der Blockroutinen
   J. Rathlev, Nov. 1995
      letzte nderung: Jul. 2001 *)

unit RwBlock;

interface

uses Dos;

const
  BufLen = 1024;
  RecMax = 2048;
  LF = 10;
  CR = 13;

type
  BufTyp = record
    BFile      : file;
    BCnt,BMax  : word;
    Buffer     : array[1..BufLen] of byte;
    end;

(* Datei lschen *)
procedure BlkErase (var Buf : BufTyp);

(* Datei umbenennen *)
procedure BlkRename (var Buf : BufTyp;
                     Name    : string);

(* Dateigre in Bytes *)
function BlkFileSize (var Buf : BufTyp) : longint;

(* Datei zum Lesen ffnen *)
function RdBlkOpen (var Buf : BufTyp;
                    Name    : PathStr) : boolean;

(* Datei an Anfang setzen *)
procedure RdBlkReset (var Buf : BufTyp);

(* Positionszeiger setzen *)
function RdBlkSeek (var Buf : BufTyp;
                    Position: longint) : boolean;

(* Datei zum Lesen schlieen *)
procedure RdBlkClose (var Buf : BufTyp);

(* auf Dateiende beim Lesen prfen *)
function RdBlkEof (var Buf : BufTyp) : boolean;

(* ein Byte einlesen *)
function RdBlkByte (var Buf : BufTyp) : byte;

(* ein Zeichen einlesen *)
function RdBlkChar (var Buf : BufTyp) : char;

(* ein Wort einlesen *)
function RdBlkWord (var Buf : BufTyp) : word;

(* ein Longint einlesen *)
function RdBlkLongint (var Buf : BufTyp) : longint;

(* Record var. Lnge einlesen *)
procedure RdBlkRec (var Buf    : BufTyp;
                    var OutVar;
                    Len        : word);

(* String einlesen *)
function RdBlkString (var Buf : BufTyp) : string;

(* Datei zum Schreiben ffnen *)
procedure WrBlkOpen (var Buf : BufTyp;
                     Name    : PathStr);

(* an Datei anhngen *)
procedure WrBlkAppend (var Buf : BufTyp;
                       Name    : PathStr);

(* Buffer in Datei schreiben *)
procedure WrBlkFlush (var Buf : BufTyp);

(* Datei zum Schreiben schlieen *)
procedure WrBlkClose (var Buf : BufTyp);

(* ein Byte schreiben *)
procedure WrBlkByte (var Buf : BufTyp;
                     b       : byte);

(* ein Zeichen schreiben *)
procedure WrBlkChar (var Buf : BufTyp;
                     c       : char);

(* ein Wort schreiben *)
procedure WrBlkWord (var Buf : BufTyp;
                     w       : word);

(* ein Longint schreiben *)
procedure WrBlkLongint (var Buf : BufTyp;
                        w       : longint);

(* Record var. Lnge schreiben *)
procedure WrBlkRec (var Buf    : BufTyp;
                    var OutVar;
                    Len        : word);

(* String schreiben *)
procedure WrBlkString (var Buf : BufTyp;
                       s       : string);

(* String mit CR schreiben *)
procedure WrBlkStringLn (var Buf : BufTyp;
                         s       : string);

implementation
{ ---------------------------------------------------------------- }
(* ein Byte einlesen *)
function RdBlkByte (var Buf : BufTyp) : byte;
begin
  with Buf do begin
    if (BCnt=0) and (BMax=BufLen) then begin
      BlockRead (BFile,Buffer,BufLen,BMax);
      end;
    if (BCnt<BMax) then begin
      inc(BCnt); RdBlkByte:=Buffer[BCnt];
      if (BCnt>=BMax) and (BMax=BufLen) then BCnt:=0;
      end
    else RdBlkByte:=0;
    end;
  end;

{ ---------------------------------------------------------------- }
(* ein Zeichen einlesen *)
function RdBlkChar (var Buf : BufTyp) : char;
begin
  RdBlkChar:=chr(RdBlkByte(Buf));
  end;

{ ---------------------------------------------------------------- }
(* ein Wort einlesen *)
function RdBlkWord (var Buf : BufTyp) : word;
var
  b : array [1..2] of byte;
  w : word absolute b;
begin
  b[1]:=RdBlkByte(Buf);
  b[2]:=RdBlkByte(Buf);
  RdBlkWord:=w;
  end;

(* ------------------------------------------------------------------------ *)
(* ein Longint einlesen *)
function RdBlkLongInt (var Buf : BufTyp) : longint;
var
  b : array [1..4] of byte;
  w : longint absolute b;
begin
  b[1]:=RdBlkByte(Buf); b[2]:=RdBlkByte(Buf);
  b[3]:=RdBlkByte(Buf); b[4]:=RdBlkByte(Buf);
  RdBlkLongInt:=w;
  end;

(* ------------------------------------------------------------------------ *)
(* Record var. Lnge einlesen *)
procedure RdBlkRec (var Buf    : BufTyp;
                    var OutVar;
                    Len        : word);
var
  Rec : array [1..RecMax] of byte absolute OutVar;
  i : word;
begin
  for i:=1 to Len do Rec[i]:=RdBlkByte(Buf);
  end;

(* ------------------------------------------------------------------------ *)
(* String bis CR, LF oder <0> einlesen *)
function RdBlkString (var Buf : BufTyp) : string;
var
  s   : string;
  bs  : array [0..255] of byte absolute s;
  b,i : byte;
  ende,
  ok  : boolean;
begin
  i:=0; ok:=false;
  repeat
    ende:=RdBlkEof (Buf);
    if not ende then begin
      b:=RdBlkByte (Buf);
      ok:=(b=CR) or (b=LF) or (b=0) or (i=255);
      if not ok then begin
        inc(i); bs[i]:=b;
        end;
      end;
    until ende or (ok and (i>0));
  bs[0]:=i;
  RdBlkString:=s;
  end;

{ ---------------------------------------------------------------- }
(* ein Byte schreiben *)
procedure WrBlkByte (var Buf : BufTyp;
                     b       : byte);
begin
  with Buf do begin
    if BCnt=BufLen then begin
      BlockWrite (BFile,Buffer,BufLen);
      BCnt:=0;
      end;
    inc(BCnt); Buffer[BCnt]:=b;
    end;
  end;

{ ---------------------------------------------------------------- }
(* ein Zeichen schreiben *)
procedure WrBlkChar (var Buf : BufTyp;
                     c       : char);
begin
  WrBlkByte(Buf,ord(c));
  end;

{ ---------------------------------------------------------------- }
(* ein Wort schreiben *)
procedure WrBlkWord (var Buf : BufTyp;
                     w       : word);
var
  b : array [1..2] of byte absolute w;
begin
  WrBlkByte(Buf,b[1]);
  WrBlkByte(Buf,b[2]);
  end;

(* ------------------------------------------------------------------------ *)
(* ein Longint schreiben *)
procedure WrBlkLongint (var Buf : BufTyp;
                        w       : longint);
var
  b : array [1..4] of byte absolute w;
begin
  WrBlkByte(Buf,b[1]); WrBlkByte(Buf,b[2]);
  WrBlkByte(Buf,b[3]); WrBlkByte(Buf,b[4]);
  end;

(* ------------------------------------------------------------------------ *)
(* Record var. Lnge schreiben *)
procedure WrBlkRec (var Buf    : BufTyp;
                    var OutVar;
                    Len        : word);
var
  Rec : array [1..RecMax] of byte absolute OutVar;
  i : word;
begin
  for i:=1 to Len do WrBlkByte(Buf,Rec[i]);
  end;

(* ------------------------------------------------------------------------ *)
(* String schreiben *)
procedure WrBlkString (var Buf : BufTyp;
                       s       : string);
var
  sl  : byte absolute s;
  i   : byte;
begin
  for i:=1 to sl do WrBlkByte (Buf,ord(s[i]));
  end;

(* ------------------------------------------------------------------------ *)
(* String mit CR schreiben *)
procedure WrBlkStringLn (var Buf : BufTyp;
                         s       : string);
begin
  WrBlkString (Buf,s); WrBlkByte(Buf,CR); WrBlkByte(Buf,LF);
  end;

{ ---------------------------------------------------------------- }
(* Datei zum Lesen ffnen *)
function RdBlkOpen (var Buf : BufTyp;
                    Name    : PathStr) : boolean;
begin
  with Buf do begin
    assign (BFile,Name);
    {$I-} reset(BFile,1); {$I+}
    RdBlkOPen:=IOResult=0;
    BCnt:=0; BMax:=BufLen;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Datei an Anfang setzen *)
procedure RdBlkReset (var Buf : BufTyp);
begin
  with Buf do begin
    reset(BFile,1);
    BCnt:=0; BMax:=BufLen;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Positionszeiger setzen *)
function RdBlkSeek (var Buf : BufTyp;
                    Position: longint) : boolean;
begin
  with Buf do begin
    {$I-} seek(BFile,Position); {$I+}
    RdBlkSeek:=IOResult=0;
    BCnt:=0; BMax:=BufLen;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Datei zum Lesen schlieen *)
procedure RdBlkClose (var Buf : BufTyp);
begin
  close (Buf.BFile);
  end;

{ ---------------------------------------------------------------- }
(* auf Dateiende beim Lesen prfen *)
function RdBlkEof (var Buf : BufTyp) : boolean;
begin
  with Buf do RdBlkEof:=(BMax<BufLen) and (BCnt=BMax);
  end;

{ ---------------------------------------------------------------- }
(* Datei zum Schreiben ffnen *)
procedure WrBlkOpen (var Buf : BufTyp;
                     Name    : PathStr);
begin
  with Buf do begin
    assign (BFile,Name); rewrite (BFile,1);
    BCnt:=0;
    end;
  end;

{ ---------------------------------------------------------------- }
(* an Datei anhngen *)
procedure WrBlkAppend (var Buf : BufTyp;
                       Name    : PathStr);
begin
  with Buf do begin
    assign (BFile,Name);
    {$I-} reset(BFile,1); {$I+}
    if IoResult=0 then seek(BFile,FileSize(BFile))
    else rewrite (BFile,1);
    BCnt:=0;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Buffer in Datei schreiben *)
procedure WrBlkFlush (var Buf : BufTyp);
begin
  with Buf do begin
    if BCnt>0 then BlockWrite (BFile,Buffer,BCnt);
    BCnt:=0;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Datei zum Schreiben schlieen *)
procedure WrBlkClose (var Buf : BufTyp);
begin
  with Buf do begin
    if BCnt>0 then BlockWrite (BFile,Buffer,BCnt);
    close (BFile);
    end;
  end;

{ ---------------------------------------------------------------- }
(* Datei lschen *)
procedure BlkErase (var Buf : BufTyp);
begin
  with Buf do begin
    {$I-} reset(BFile,1); {$I+}
    if IoResult=0 then begin
       close (BFile); erase (BFile);
       end;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Datei umbenennen *)
procedure BlkRename (var Buf : BufTyp;
                     Name    : string);
begin
  with Buf do begin
    {$I-} reset(BFile,1); {$I+}
    if IoResult=0 then begin
      close (BFile); rename (BFile,Name);
      end;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Dateigre in Bytes *)
function BlkFileSize (var Buf : BufTyp) : longint;
begin
  BlkFileSize:=FileSize(Buf.BFile);
  end;

end.
